@@ -13,7 +13,6 @@ class Model:
 
     xvars: List[sympy.Symbol]
     yvars: List[sympy.Symbol]
-    ymvars: List[sympy.Symbol]
     equations: Tuple[sympy.Expr, ...]
     final_var: sympy.Symbol
 
@@ -41,7 +40,6 @@ class Model:
         )
 
         # more dimensions
-        self.pdim = len(self.ymvars)
         self.qxdim = utils.count_nonzero(self.idx)
         self.qydim = utils.count_nonzero(self.idy)
         self.qdim = self.qxdim + self.qydim
@@ -56,7 +54,9 @@ class Model:
         assert xdat.ndim == 2, f"xdat must be m*tau (is {xdat.ndim}-dimensional)"
         assert xdat.shape[0] == self.mdim, f"xdat must be m*tau (is {xdat.shape})"
         bias_dat = [bias if i == bias_ind else 0 for i in range(len(self.biases))]
-        yhat = np.array([self._model_lam(*xval, *bias_dat) for xval in xdat.T]).T
+        yhat = np.array(
+            [self._model_lam(*xval, *bias_dat) for xval in xdat.T], dtype=np.float64
+        ).T
         assert yhat.shape == (self.ndim, xdat.shape[1])
         return yhat
 
@@ -64,12 +64,8 @@ class Model:
         mx_lam, my_lam = self.m_pair
 
         # numeric direct effects since no sympy algebraic derivative
-        mx_theo = utils.replace_heaviside(
-            np.array(mx_lam(xval)), self.xvars, xval
-        )  # yyy
-        my_theo = utils.replace_heaviside(
-            np.array(my_lam(xval)), self.xvars, xval
-        )  # yyy
+        mx_theo = np.array(mx_lam(xval)).astype(np.float64)
+        my_theo = np.array(my_lam(xval)).astype(np.float64)
 
         # total and final effects
         ex_theo, ey_theo = utils.total_effects_alg(mx_theo, my_theo, self.edx, self.edy)
@@ -102,6 +98,8 @@ class Model:
             [[sympy.diff(eq, yvar) for yvar in self.yvars] for eq in self.equations]
         )
 
+        mx_alg = utils.replace_heaviside(mx_alg)
+        my_alg = utils.replace_heaviside(my_alg)
         mx_alg[mx_alg == 0] = float("NaN")
         my_alg[my_alg == 0] = float("NaN")
 
